

proc DefineConfig {s} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

#    format of module configuration record is   vsn type npatt datamask submask shift
#      where type is silena=2, lecroy=3, ortec=4, clover=5, c1011=7
#            npatt is number of pattern words at start of module data   (silena=1, others=0)
#            data mask is required to extract adc data from the channel data
#            shift is applied to the channel data to extract the sub address
#            submask is then required to extract the sub address

   write-register EXEC.Parameter $s
   write-register EXEC.Action 13
   write-register EXEC.Sig1 0
   wait_for_completion "Define Configuration"
}

proc define_fr48reg_module {module vsn} {DefineConfig "$vsn 1 0 0x0fff 3 12"}

proc create_fr48reg_spectra {module vsn} {}

proc define_silena_module {module vsn} {DefineConfig "$vsn 2 1 0x0fff 7 12"}    ;#  silena 4418

proc create_silena_spectra {module vsn} {
   global SpectrumServer

#     silena 4418

    for {set j 0} {$j <= 7} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j for vsn=$vsn and sub address=$j"
      }
    }
}

proc define_lecroy_module {module vsn} {DefineConfig "$vsn 3 0 0x07ff 0xf 11"}    ;#   lecroy 4300

proc create_lecroy_spectra {module vsn} {
   global SpectrumServer

#     lecroy 4300

     for {set j 0} {$j <= 15} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 2048 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j for vsn=$vsn and sub address=$j"
      }
    }
}

proc define_ortec_module {module vsn} {DefineConfig "$vsn 4 0 0x1fff 3 13"}   ;#  ortec 413

proc create_ortec_spectra {module vsn} {
   global SpectrumServer

#     ortec 413

    for {set j 0} {$j <= 3} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 8192 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j for vsn=$vsn and sub address=$j"
      }
    }
}

proc map_spectra {} {
   global EGreport RDOUTEnable RDOUThdwre

   if {$RDOUTEnable == 0} then {return}

   if {[map_?_needed] == 1} {return}

   set-busy
   set-footer "Mapping VXI spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1

   set l [spectra-names]
   set num 0
   foreach item $l {
       incr num
       set name [second $item]
       if {"$name" == "Stat"} continue
       if {"$name" == "Rate"} continue
       map_spectrum "$name 256 0"
       if {[expr ($num/10)*10] == $num} {set-footer "mapped spectrum $name"}
#       set-footer "mapped spectrum $name"
   }

   map_not_needed

   set EGreport $EGreport_SAVE

   set-footer "mapped [llength $l] VXI spectra"
   clear-busy
}

proc click-histmode {w v} {
    global HistMode
    set  HistMode 0
    redisplay-histmode
}
